Rem
bbdoc:	Stores a sequence of elements
End Rem
Type Array Extends Object Abstract Final

	Rem
	bbdoc:	The number of elements the array holds
	about:	<i>read-only</i>	
	End Rem
	Field length:Int
	
	Rem
	bbdoc:	The number of dimenions
	about:	<i>read-only</i>
	End Rem
	Field numberOfDimensions:Int
	
	Rem
	bbdoc:	The size of the array's data in bytes
	about:	<i>read-only</i>
	End Rem
	Field sizeMinusHeader:Int

	Rem
	bbdoc:	Gives a short description of the array
	returns: A string describing the the array
	about:	<p>The description string has the structure:</p>
			<code>(Array) $TYPE[$DIMENSIONS]</code>
	End Rem
	Method toString:String()
	End Method
	
	Rem
	bbdoc:	Compares two arrays
	End Rem
	Method compare:Int(other:Array)
	End Method
	
	Rem
	bbdoc:	Checks if the array equals another one
	returns: <tt>True</tt> if the @other array equals this one, otherwise <tt>False</tt>.
	End Rem
	Method equals:Int(other:Array)
	End Method
	
	Rem
	bbdoc:	Clones the array
	returns: A new array with the same values
	about:	Primitive type array's (<tt>Int[]</tt>, <tt>Float[]</tt> etc) data will be copied by value.
			The data of complex type arrays (<tt>Object[]</tt>, <tt>String[]</tt> etc) however, will be copied by reference.
	End Rem
	Method clone:Array()
	End Method


	Rem
	bbdoc:	Specifies a <tt>Byte</tt> array
	End Rem
	Const TYPE_BYTE:Int		= 0
	
	Rem
	bbdoc:	Specifies a <tt>Short</tt> array
	End Rem
	Const TYPE_SHORT:Int	= 1
	
	Rem
	bbdoc:	Specifies an <tt>Int</tt> array
	End Rem
	Const TYPE_INT:Int		= 2
	
	Rem
	bbdoc:	Specifies a <tt>Long</tt> array
	End Rem
	Const TYPE_LONG:Int		= 3
	
	Rem
	bbdoc:	Specifies a <tt>Float</tt> array
	End Rem
	Const TYPE_FLOAT:Int	= 4
	
	Rem
	bbdoc:	Specifies a <tt>Double</tt> array
	End Rem
	Const TYPE_DOUBLE:Int	= 5
	

	Rem
	bbdoc:	Creates a new array from a C-style buffer
	returns: A new array object with the given type
	param:	buf A block of numeric data
	param:	size The size of the buffer in bytes
	param:	typekey The type of the data stored in @buf
	param:	dims The size of each dimension
	about:	<p>Only numeric type C-style arrays can be used as input.</p>
	
			<p>Returns <tt>Null</tt> if @buf is <tt>NULL</tt>, @size is negative or too small
			or the @typekey is invalid.</p>
	End Rem
	Function FromCArray:Array(buf:Byte Ptr, size:Int, typekey:Int = Array.TYPE_INT, dims:Int[] = Null)
	End Function
	
	Rem
	bbdoc:	Converts the array to a sequence of raw numeric data.
	returns: A newly allocated block of memory containing the array's data.
	about:	<p>Only numeric arrays can be converted to a C-style array,
			all others will return <tt>NULL</tt>.</p>
	
			<div class="bbdoc-note">
				<p><strong>Note:</strong> Remember to free the returned memory block with #MemFree.</p>
			</div>
	End Rem
	Method toCArray:Byte Ptr()
	End Method	
	

	Rem
	bbdoc:	States that an element or array could not be found
	about:	Used by #find, #findInt, #findLong, #findFloat, #findDouble, #findObject and #findString, 
	End Rem
	Const NOT_FOUND:Int = -1

	Rem
	bbdoc:	Finds the first occurance of a subarray in the array
	returns: The position of the first matching subarray, or #NOT_FOUND.
	End Rem
	Method find:Int(subarray:Array, start_index:Int = 0)
	End Method

	Rem
	bbdoc:	Finds the first occurance of an <tt>Int</tt> value in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findInt:Int(number:Int, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of a <tt>Long</tt> value in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findLong:Int(number:Long, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of a <tt>Float</tt> value in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findFloat:Int(number:Float, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of a <tt>Double</tt> value in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findDouble:Int(number:Double, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of an object in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findObject:Int(obj:Object, start_index:Int = 0)
	End Method
	
	Rem
	bbdoc:	Finds the first occurance of a string in the array
	returns: The position of the first matching element, or #NOT_FOUND.
	End Rem
	Method findString:Int(str:String, start_index:Int = 0)
	End Method
	
	
	
	Rem
	bbdoc:	Checks if the array starts with another one
	returns: <tt>True</tt> if the array starts with @subarray, otherwise <tt>False</tt>.
	End Rem
	Method startsWith:Int(subarray:Array)
	End Method
	
	Rem
	bbdoc:	Checks if the array ends with another one
	returns: <tt>True</tt> if the array ends with @subarray, otherwise <tt>False</tt>.
	End Rem
	Method endsWith:Int(subarray:Array)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains another one.
	returns: <tt>True</tt> if the array contains the @subarray at least once, otherwise <tt>False</tt>.
	End Rem
	Method contains:Int(subarray:Array)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains an <tt>Int</tt> element with the given value.
	returns: <tt>True</tt> if the array contains the @number at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsInt:Int(number:Int)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains a <tt>Long</tt> element with the given value.
	returns: <tt>True</tt> if the array contains the @number at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsLong:Int(number:Long)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains a <tt>Float</tt> element with the given value.
	returns: <tt>True</tt> if the array contains the @number at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsFloat:Int(number:Float)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains a <tt>Double</tt> element with the given value.
	returns: <tt>True</tt> if the array contains the @number at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsDouble:Int(number:Double)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains an object.
	returns: <tt>True</tt> if the array contains the @obj at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsObject:Int(obj:Object)
	End Method
	
	Rem
	bbdoc:	Checks if the array contains a string.
	returns: <tt>True</tt> if the array contains the @str at least once, otherwise <tt>False</tt>.
	End Rem
	Method containsString:Int(str:String)
	End Method


	
	Rem
	bbdoc:	Reverses the array's elements
	returns: A reversed version of the array.
	about:	Empty or single element arrays are returned unaltered.
	End Rem
	Method reverse:Array()
	End Method
	
	Rem
	bbdoc:	Rotates the array's elements
	End Rem
	Method rotate:Array(count:Int = 1)
	End Method
	
	Rem
	bbdoc:	Sorts the array's elements
	End Rem
	Method sort(ascending:Int = True)
	End Method
	
	
	Rem
	bbdoc:	Filters the array's elements using a callback function
	returns: A new array including only the filtered values
	about:	If the @filter function's boolean return value is <tt>True</tt>, the element is
			included in the output array. Otherwise it won't be.
	End Rem
	Method filterInt:Int[](filter:Int(value:Int))
	End Method
		
	
	Rem
	bbdoc:	Gives the array's dimensions
	returns: An <tt>Int</tt> array containing the size of each dimension
	about:	If you are only interested in how many dimensions an array holds,
			use the #numberOfDimensions property.
	End Rem
	Method dimensions:Int[]()
	End Method

End Type
